from arielbot.plugins.Core.ariel_database import DataManager
from nonebot.adapters.onebot.v11 import GroupMessageEvent


async def bot_is_active(event:GroupMessageEvent) -> bool:
    async with DataManager() as m:
        result = await m.select_bot_status((event.self_id,event.group_id))
        if not result:
            await m.insert_bot_status((event.self_id,event.group_id,1,1))
            return True
        return (result[0] and result[1])



import qrcode
import time
import skia
import pickle
from io import BytesIO
from nonebot import logger
from arielbot.plugins.Core.ariel_bili import Login,UserInfo
from arielbot.plugins.Core.ariel_database import DataManager
from urllib.parse import urlsplit, parse_qs
from nonebot.adapters.onebot.v11 import Bot
from nonebot.adapters.onebot.v11 import MessageSegment,GroupMessageEvent


class LoginTools:
    
    async def login_handle(self,bot:Bot,event:GroupMessageEvent):
        login = Login()
        scan_url =  await login.get_qrcode_key()
        if scan_url is None:
            await bot.send(event=event,message=MessageSegment.text("获取扫码链接失败"))
            return
        qrcode_buffer = BytesIO()
        qr = qrcode.QRCode()
        qr.add_data(scan_url)
        img = qr.make_image(image_factory=qrcode.image.pure.PyPNGImage)
        img.save(qrcode_buffer)
        await bot.send(event,message=MessageSegment.image(qrcode_buffer))
        while True:
            scan_result = await login.check_scan_result()
            if scan_result is None or scan_result["code"]==86038:
                await bot.send(event,"登陆失败")
                break
            if scan_result["code"] == 0:
                cookies = await self.__parse_cookie(scan_result)
                if cookies is None:
                    await bot.send(event,MessageSegment.text("cookie 解析失败"))
                    break
                async with DataManager() as d:
                    await d.insert_cookie((pickle.dumps(cookies),scan_result["refresh_token"]))
                break
            time.sleep(3)

    
    async def __parse_cookie(self,data):
        try:
            query_str = urlsplit(data["url"]).query
            params = parse_qs(query_str)
            cookies = {k: v[0] for k, v in params.items()}
            cookies.pop("gourl")
            return cookies
        except Exception as e:
            logger.error(e)
            return None

class PublicSubTools:
    def __init__(self,uid):
        self.uid = uid
    
    async def check_uid_info(self):
        uinfo = UserInfo()
        data = await uinfo.get_user_info_by_uid(self.uid)
        return data
    
    async def check_uid_in_group(self,bot:int,groupId:int):
        async with DataManager() as m:
            return await m.select_sub_chennal((self.uid,groupId,bot))
    
    async def check_uid_has_sub(self):
        async with DataManager() as m:
            return await m.select_sub_target(self.uid)
    
    async def follow_user(self,uid,act):
        uinfo = UserInfo()
        return await uinfo.change_follow_status(uid,act)
        

    
class AddSubTools(PublicSubTools):
    def __init__(self, uid):
        super().__init__(uid)
        
    async def add_sub_processor(self,event:GroupMessageEvent):
        check_sub_result = await self.check_uid_has_sub()
        if check_sub_result:
            check_uid_in_group_result = await self.check_uid_in_group(event.self_id,event.group_id)
            if check_uid_in_group_result:
                if check_uid_in_group_result[0]==0 or check_uid_in_group_result[1]==0:
                    async with DataManager() as m:
                        await m.update_sub_chennal((1,1,self.uid,event.group_id,event.self_id))
                    return f"成功添加订阅 --> {check_sub_result[0]}({self.uid})"
                else:
                    return f"本群已订阅过 --> {check_sub_result[0]}({self.uid})"
            else:
                async with DataManager() as m:
                    await m.insert_sub_chennal((self.uid,event.group_id,event.self_id))
                return f"成功添加订阅 --> {check_sub_result[0]}({self.uid})"
        else:
            uid_info = await self.check_uid_info()
            if isinstance(uid_info,str):
                return uid_info
            if uid_info["following"] != True:
                follow_result =  await self.follow_user(self.uid,1)
                if not follow_result:
                    return "添加订阅失败"
            async with DataManager() as m:
                await m.insert_sub_target((self.uid,uid_info["card"]["name"],0))
                await m.insert_sub_chennal((self.uid,event.group_id,event.self_id))
            return f"成功添加订阅 --> {uid_info["card"]["name"]}({self.uid})"            

class DelSubTools(PublicSubTools):
    def __init__(self, uid):
        super().__init__(uid)
    
    
    async def del_sub_processor(self,event:GroupMessageEvent):
        check_uid_in_group_result = await self.check_uid_in_group(event.self_id,event.group_id)
        if not check_uid_in_group_result:
            return f"本群没有订阅 --> {self.uid}"
        else:
            async with DataManager() as m:
                await m.update_sub_chennal((0,0,self.uid,event.group_id,event.self_id))
                uid_info = await m.select_sub_target(self.uid)
            return f"成功删除订阅 --> {uid_info[0]}({self.uid})"

class UpdateSubTools(PublicSubTools):
    def __init__(self, uid):
        super().__init__(uid)
        
    async def update_sub_handler(self,event:GroupMessageEvent,live_active:int=None, dyn_active:int = None):
        check_uid_in_group_result = await self.check_uid_in_group(event.self_id,event.group_id)
        if not check_uid_in_group_result:
            return f"本群没有订阅 --> {self.uid}"
        old_live_active = check_uid_in_group_result[0]
        old_dyn_active = check_uid_in_group_result[1]
        async with DataManager() as m:
            if live_active is None:
                await m.update_sub_chennal((old_live_active,dyn_active,self.uid,event.group_id,event.self_id))
                return "开启动态推送成功" if dyn_active==1 else "关闭动态推送成功"
            else:
                await m.update_sub_chennal((live_active,old_dyn_active,self.uid,event.group_id,event.self_id))
                return "开启直播推送成功" if live_active==1 else "关闭直播推送成功"
                
class UpdateBotStatusTools:
    async def update_bot_status_processor(self,event:GroupMessageEvent,status_new):
        async with DataManager() as m:
            result = await m.select_bot_status((event.self_id,event.group_id))
            if not result:
                await m.insert_bot_status((event.self_id,event.group_id,status_new,1))
                return "bot已开启" if status_new==1 else "bot已关闭"
            if status_new == result[0]:
                return None if status_new==0 else "bot已经为开启状态"
            else:
                await m.update_bot_push_status((status_new,event.self_id,event.group_id))
                return "bot关闭成功" if status_new==0 else "bot开启成功"  
    
    @staticmethod
    async def update_bot_active_processor(bot_active_status):
        async with DataManager() as m:
            await m.updata_bot_active_status(bot_active_status)   
    
    @staticmethod
    async def shutdown_all_bot():
        async with DataManager() as m:
            result = await m.select_all_bot()
            if not result:
                return
            for i in result:
                logger.info(f"关闭机器人：{i[0]}")
                await m.updata_bot_active_status((0,i[0]))
            

class SubListTools:

    async def get_sub_list_data(self,event:GroupMessageEvent):
        async with DataManager() as m:
            all_data = await m.select_sub_list((event.self_id,event.group_id))
        if not all_data:
            return MessageSegment.text("本群订阅列表为空")
        sub_list_img = await self.__make_sub_img(all_data)
        return sub_list_img
        

    async def __make_sub_img(self,sub_data:list):
        if len(sub_data)<=8:
            img_height = 540
        else:
            img_height = 60*(len(sub_data)+1)
        surface = skia.Surface(1000,img_height)
        canvas = surface.getCanvas()
        canvas.clear(skia.ColorWHITE)
        paint = skia.Paint(
                Color=skia.ColorBLACK,         
                StrokeWidth=1,                 
                AntiAlias=True,
                Style=skia.Paint.kStroke_Style
            )
        for i in range(250,1000,250):
            canvas.drawLine(i, 0, i, img_height, paint)
        for i in range(60,540,60):
            rect = skia.Rect.MakeXYWH(0, 0, 1000, i)
            canvas.drawRect(rect, paint)
        typeface = skia.FontMgr().matchFamilyStyleCharacter("Noto Sans CJK SC",skia.FontStyle().Normal(),["zh", "en"],ord(sub_data[0][1][0]),)
        logger.info(typeface.getFamilyName())
        paint.setStyle(skia.Paint.Style.kFill_Style)
        font = skia.Font(typeface, 16)
        metrics = font.getMetrics()
        baseline_height = round(abs(metrics.fAscent)) 
        sub_data.insert(0,("UID","昵称","动态推送","直播推送"))
        for i,content in enumerate(sub_data):
            
            for j in  range(len(content)):
                
                if content[j]==1:
                    text = "开"
                    paint.setARGB(255,0,0,0)
                elif content[j] ==0:
                    text = "关"
                    paint.setARGB(255,255,0,0)
                else:
                    paint.setARGB(255,0,0,0)
                    text = content[j]
                blob = skia.TextBlob(text, font)
                text_len = font.measureText(text)
                canvas.drawTextBlob(blob, (125+250*j-int(text_len/2)), 30+i*60+int(baseline_height/2), paint)
        img = skia.Image.fromarray(canvas.toarray(colorType=skia.ColorType.kRGBA_8888_ColorType), colorType=skia.ColorType.kRGBA_8888_ColorType)
        img_buffer = BytesIO()
        img.save(img_buffer)
        return MessageSegment.image(img_buffer)
        
        
        
                
        
        



    
    


import nonebot
from nonebot.adapters.onebot.v11 import Adapter
from os import path, getcwd, listdir
import sys

sys.path.append(path.join(getcwd(), "plugins"))
nonebot.init()
nonebot.load_plugin("arielbot.plugins.Core")
nonebot.load_all_plugins([i for i in listdir(path.join(getcwd(), "plugins"))], [])
driver = nonebot.get_driver()
driver.register_adapter(Adapter)
app = nonebot.get_asgi()
config = nonebot.get_driver().config



import re
import time
import httpx
import pickle
import binascii
from nonebot import logger
from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP
from http.cookies import SimpleCookie
from datetime import datetime,timezone
from arielbot.plugins.Core.ariel_database import DataManager

class CookieManager:
    def __init__(self):
        self.cookie = None
        self.refresh_token = None
        self.headers  = {
        "user-agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36 Edg/134.0.0.0"
        }
        self.key = RSA.importKey('''\
-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDLgd2OAkcGVtoE3ThUREbio0Eg
Uc/prcajMKXvkCKFCWhJYJcLkcM2DKKcSeFpD/j6Boy538YXnR6VhcuUJOhH2x71
nzPjfdTcqMz7djHum0qSZA0AyCBDABUqCrfNgCiJ00Ra7GmRj+YCK1NJEuewlb40
JNrRuoEUXpabUzGB8QIDAQAB
-----END PUBLIC KEY-----''')
        

    async def get_cookie(self):
        async with DataManager() as m:
            result  = await m.select_cookie()
        if result is None:
            logger.info("未登录")
            return            
        self.refresh_token = result[1]
        self.cookie = pickle.loads(result[0])
        await self.__check_expire()
    
    async def __check_expire(self):
        if self.cookie is None:
            return
        cookie_expire = self.cookie["Expires"]
        now = int(time.time())
        if int(cookie_expire) - now > 3600:
            return
        await self.__refresh_cookie()
        
    async def __refresh_cookie(self):
        correspond_path = await self.__get_correspond_path()
        if correspond_path is None:
            return
        refresh_csrf =  await self.__get_refresh_csrf(correspond_path)
        if refresh_csrf is None:
            return
        new_data =  await self.__get_new_cookie(refresh_csrf)
        if new_data is None:
            return
        self.cookie = new_data[0]
        async with DataManager() as m:
            await m.update_cookie((pickle.dumps(self.cookie), new_data[1],self.refresh_token))
        self.refresh_token = new_data[1]
     
    async def __get_correspond_path(self):
        params = {"csrf":self.cookie["bili_jct"]}
        url = "https://passport.bilibili.com/x/passport-login/web/cookie/info"
        try:
            response = httpx.get(url,headers=self.headers,params=params,cookies=self.cookie)
            cipher = PKCS1_OAEP.new(self.key, SHA256)
            encrypted = cipher.encrypt(f'refresh_{response.json()["data"]["timestamp"]}'.encode())
            return binascii.b2a_hex(encrypted).decode()
        except Exception as e:
            logger.error(e)
            return None

    async def __get_refresh_csrf(self,correspond_path):
        url = f"https://www.bilibili.com/correspond/1/{correspond_path}"
        try:
            respoese = httpx.get(url,headers=self.headers,cookies=self.cookie)
            pattern = re.compile(r'<div\s+id\s*=\s*["\']1-name["\']\s*>(.*?)</div>',flags=re.DOTALL)
            match = pattern.search(respoese.text)
            return match.group(1).strip()
        except Exception as e:
            logger.error("get refresh_csrf error")
            logger.error(e)
            return None

    async def __get_new_cookie(self,refresh_csrf):
        url = "https://passport.bilibili.com/x/passport-login/web/cookie/refresh"
        data = {
            "csrf":self.cookie["bili_jct"],
            "refresh_csrf":refresh_csrf,
            "source":"main_web",
            "refresh_token":self.refresh_token
        }
        try:
            response = httpx.post(url,headers=self.headers,data=data,cookies=self.cookie)
            new_refresh_token  = response.json()["data"]["refresh_token"]
            new_cookies = response.headers.get_list("Set-Cookie")
            parsed_cookies = [await self.__parse_cookie_attributes(c) for c in  new_cookies]
            expires = parsed_cookies[0]["expires"]
            dt = datetime.strptime(expires, "%a, %d %b %Y %H:%M:%S GMT").replace(tzinfo=timezone.utc)
            new_cookie = {}
            timestamp = int(dt.timestamp())
            for i in parsed_cookies:
                new_cookie[i["name"]] = i["value"]
            new_cookie.update({"Expires":str (timestamp)})
            return (new_cookie,new_refresh_token)
        except Exception as e:
            logger.error("get new cookie error")
            logger.error(e)
            return None

    async def __parse_cookie_attributes(self,cookie_str: str) -> dict:
        cookie = SimpleCookie()
        cookie.load(cookie_str)
        parsed = {}
        for key, morsel in cookie.items():
            parsed.update({
                "name": key,
                "value": morsel.value,
                "expires": morsel.get("expires", None)
            })
        return parsed


if __name__ == "__main__":
    manager = CookieManager()
    


import time
import httpx
import urllib.parse
from hashlib import md5
from nonebot import logger
from functools import reduce
from typing import Optional,List,Union
from arielbot.plugins.Core.ariel_cookie import CookieManager
from dynamicadaptor.Message import RenderMessage
from dynamicadaptor.DynamicConversion import formate_message


class Login:
    def __init__(self):
        self.qrcode_key = None
        self.headers  = {
        "user-agent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/134.0.0.0 Safari/537.36 Edg/134.0.0.0"
        }

    async def check_scan_result(self) -> Optional[dict]:
        """获取登陆二维码扫描结果

        :return: {
                        "code": 0,
                        "message": "0",
                        "ttl": 1,
                        "data": {
                            "url": "https://passport.biligame.com/crossDomain?DedeUserID=&DedeUserID__ckMd5=*&Expires=*&SESSDATA=*&bili_jct=**&gourl=*",
                            "refresh_token": "***",
                            "timestamp": 1662363009601,
                            "code": 0,
                            "message": ""
                        }
                    }
        :rtype: Optional[dict]
        """

        url = "https://passport.bilibili.com/x/passport-login/web/qrcode/poll"
        params = {
            "qrcode_key":self.qrcode_key
        }
        try:
            response = httpx.get(url,headers=self.headers,params=params)
            response.raise_for_status()
            return response.json()["data"]
        except Exception as e:
            logger.error(e)
            return None

    async def get_qrcode_key(self):
        url = "https://passport.bilibili.com/x/passport-login/web/qrcode/generate"
        try:
            response = httpx.get(url,headers=self.headers)
            response.raise_for_status()
            self.qrcode_key = response.json()["data"]["qrcode_key"]
            return response.json()["data"]["url"]
        except Exception as e:
            logger.error(e)
            return None


class Dynamic(CookieManager):
    def __init__(self):
        super().__init__()

    async def get_dynamic_from_follow_list(self) -> Optional[List[RenderMessage]] :
        """获取关注列表的前20条动态

        :return: 格式化后的动态
        :rtype: Optional[List[RenderMessage]]
        """
        await self.get_cookie()
        if self.cookie is None:
            return None
        self.headers.update({
            "referer":"https://t.bilibili.com/?spm_id_from=333.1007.0.0",
            "origin":"https://t.bilibili.com"
        })
        url = "https://api.bilibili.com/x/polymer/web-dynamic/v1/feed/all"
        params = {
            "timezone_offset":"-480",
            "type":"all",
            "web_location":"333.1365",
            "platform":"web",
            "page":1,
            "features":"itemOpusStyle,listOnlyfans,opusBigCover,onlyfansVote,decorationCard,onlyfansAssetsV2,forwardListHidden,ugcDelete,onlyfansQaCard,commentsNewVersion",
            "x-bili-device-req-json":{"platform":"web","device":"pc"},
            "x-bili-web-req-json":{"spm_id":"333.1365"}
        }
        try:
            response = httpx.get(headers=self.headers,url=url,cookies=self.cookie,params=params)
            response.raise_for_status()
            data = response.json()
            if data["code"] != 0:
                logger.warning("get dynamic from follow list data code is not 0")
                return None
            return  [ await formate_message("web",i) for i in  data["data"]["items"]]
        except Exception as e:
            logger.warning(f"get dynamic from follow list error {e}")
            return None
            
    async def get_dynamic_from_id(self,dyn_id:str) -> Optional[RenderMessage]:
        """获取指定动态id的动态信息

        :param dyn_id: 动态id
        :type dyn_id: str
        :return: 格式化后的动态信息
        :rtype: Optional[RenderMessage]
        """
        
        await self.get_cookie()
        if self.cookie is None:
            return None
        sign = Sign()
        await sign.getWbiKeys()
        if sign.img_key is None or sign.sub_key is None:
            logger.warning("get img_key or sub_key error")
            return None
        self.headers.update({
            "referer":f"https://t.bilibili.com/{dyn_id}?spm_id_from=333.1365.list.card_time.click",
            "origin":"https://t.bilibili.com"
        })
        params = {
            "timezone_offset":"-480",
            "platform":"web",
            "gaia_source":"main_web",
            "id":dyn_id,
            "features":"itemOpusStyle,opusBigCover,onlyfansVote,endFooterHidden,decorationCard,onlyfansAssetsV2,ugcDelete,onlyfansQaCard,editable,opusPrivateVisible"
        }
        signed_params = await sign.encWbi(
            params=params,
            img_key=sign.img_key,
            sub_key=sign.sub_key
        )
        url = "https://api.bilibili.com/x/polymer/web-dynamic/v1/detail"
        try:
            response = httpx.get(headers=self.headers,url = url,params=signed_params,cookies=self.cookie)
            response.raise_for_status()
            return await formate_message("web",response.json()["data"]["item"])
        except Exception as e:
            logger.error(e)
            return None
        

class Live(CookieManager):
    def __init__(self):
        super().__init__()
    
    async def get_live_users_from_follow_list(self):
        await self.get_cookie()
        if self.cookie is None:
            return None
        self.headers.update({
            "referer":"https://t.bilibili.com/?spm_id_from=333.1007.0.0",
            "origin":"https://t.bilibili.com"
        })
        params = {
            "up_list_more":1,
            "web_location":333.1365
        }
        url = "https://api.bilibili.com/x/polymer/web-dynamic/v1/portal"
        try:
            response = httpx.get(headers=self.headers,url=url,cookies=self.cookie,params=params)
            response.raise_for_status()
            return response.json()["data"]["live_users"]
        except Exception as e:
            logger.error(e)
            return None

    async def get_room_info_by_uids(self,uids:List[Union[int, str]]):
        url = "https://api.live.bilibili.com/room/v1/Room/get_status_info_by_uids"
        data = {
            "uids":uids
        }
        try:
            response = httpx.post(url,headers=self.headers,json=data)
            response.raise_for_status()
            return response.json()["data"]
        except Exception as e:
            logger.error(e)
            return None

class UserInfo(CookieManager):
    def __init__(self):
        super().__init__()
    
    async def get_user_info_by_uid(self,uid):
        """通过uid获取信息

        Args:
            uid (int): 用户uid
        """
        
        await self.get_cookie()
        if self.cookie is None:
            return "未登录"
        url = "https://api.bilibili.com/x/web-interface/card"
        params = {
            "mid":uid,
            "photo":"true",
            "web_location":"0.0"
        }
        self.headers.update({
            "host":"api.bilibili.com",
            "origin":"https://t.bilibili.com",
            "referer":"https://t.bilibili.com/"
        })
        try:
            response = httpx.get(url,headers=self.headers,cookies=self.cookie,params=params)
            response.raise_for_status()
            if response.json()["code"] !=0:
                return "未找到相关UP信息"
            else:
                return response.json()["data"]
        except Exception as e:
            return str(e)

    async def change_follow_status(self, uid, act):
        await self.get_cookie()
        if self.cookie is None:
            return None
        url = 'https://api.bilibili.com/x/relation/modify?statistics={"appId":100,"platform":5}&x-bili-device-req-json={"platform":"web","device":"pc","spmid":"0.0"}'
        data = {
            "fid":f"{uid}",
            "act":f"{act}",
            "re_src":"11",
            "gaia_source":"web_main",
            "spmid":"0.0",
            "extend_content":'{"entity":"user","entity_id":477332594}',
            "is_from_frontend_component":"true",
            "csrf":self.cookie["bili_jct"]
        }
        try:
            response = httpx.post(url,headers=self.headers,cookies=self.cookie,data=data)
            response.raise_for_status()
            if response.json()["code"]==0:
                return True
            return None
        except Exception as e:
            logger.error(e)
            return None
    

class Sign:
    def __init__(self):
        self.mixinKeyEncTab = [
    46, 47, 18, 2, 53, 8, 23, 32, 15, 50, 10, 31, 58, 3, 45, 35, 27, 43, 5, 49,
    33, 9, 42, 19, 29, 28, 14, 39, 12, 38, 41, 13, 37, 48, 7, 16, 24, 55, 40,
    61, 26, 17, 0, 1, 60, 51, 30, 4, 22, 25, 54, 21, 56, 59, 6, 63, 57, 62, 11,
    36, 20, 34, 44, 52
]
        self.headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3',
        'Referer': 'https://www.bilibili.com/'
    }
        self.img_key = None
        self.sub_key = None
        
    async def getWbiKeys(self) -> None:
        try:
            resp = httpx.get('https://api.bilibili.com/x/web-interface/nav', headers=self.headers)
            resp.raise_for_status()
            json_content = resp.json()
            img_url: str = json_content['data']['wbi_img']['img_url']
            sub_url: str = json_content['data']['wbi_img']['sub_url']
            self.img_key = img_url.rsplit('/', 1)[1].split('.')[0]
            self.sub_key = sub_url.rsplit('/', 1)[1].split('.')[0]
        except Exception as e:
            logger.error(e)
        
    async def __getMixinKey(self, orig: str):
        '对 imgKey 和 subKey 进行字符顺序打乱编码'
        return reduce(lambda s, i: s + orig[i], self.mixinKeyEncTab, '')[:32]
    
    async def encWbi(self,params: dict, img_key: str, sub_key: str):
        '为请求参数进行 wbi 签名'
        mixin_key = await self.__getMixinKey(img_key + sub_key)
        curr_time = round(time.time())
        params['wts'] = curr_time                                   # 添加 wts 字段
        params = dict(sorted(params.items()))                       # 按照 key 重排参数
        # 过滤 value 中的 "!'()*" 字符
        params = {
            k : ''.join(filter(lambda chr: chr not in "!'()*", str(v)))
            for k, v 
            in params.items()
        }
        query = urllib.parse.urlencode(params)                      # 序列化参数
        wbi_sign = md5((query + mixin_key).encode()).hexdigest()    # 计算 w_rid
        params['w_rid'] = wbi_sign
        return params


import aiosqlite
from os import path,getcwd
from typing import Optional

class DataManager:
    def __init__(self):
        self.__conn: Optional[aiosqlite.Connection] = None
        self.__cursor:Optional[aiosqlite.Cursor] = None
        self.dbexists = True

    async def __aenter__(self):
        db_path = path.join(getcwd(),"data.sqlite")
        if not path.exists(db_path):
            self.dbexists=False
        self.__conn = await aiosqlite.connect(db_path)
        self.__cursor = await self.__conn.cursor()
        await self.__cursor.execute("BEGIN")
        if not self.dbexists:
            await self.__cursor.execute("PRAGMA foreign_keys = ON;")
            await self.__creat_subTarget_table()
            await self.__creat_subChennal_table()
            await self.__creat_botStatus_table()
            await self.__creat_cookie_table()
            await self.__creat_dynamic_table()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if exc_type is None:
            await self.__conn.commit()
        else:
            await self.__conn.rollback()
        await self.__cursor.close()
        await self.__conn.close()
        self.__cursor = None
        self.__conn = None
    
    async def __creat_subTarget_table(self):
        sql ="""
            CREATE TABLE IF NOT EXISTS subTarget (
                id INTEGER PRIMARY KEY AUTOINCREMENT, 
                nickname TEXT NOT NULL,
                uid TEXT NOT NULL UNIQUE,
                live_status INTEGER NOT NULL DEFAULT 1 CHECK(live_status IN (0, 1))
            );
            """
        
        await self.__cursor.execute(sql)
    
    async def __creat_subChennal_table(self):
        sql ="""
            CREATE TABLE IF NOT EXISTS subChennal (
                id INTEGER PRIMARY KEY AUTOINCREMENT, 
                uid TEXT NOT NULL,
                groupId INTEGER NOT NULL,
                bot INTEGER NOT NULL,
                live_active INTEGER NOT NULL DEFAULT 1 CHECK(live_active IN (0, 1)),
                dyn_active INTEGER NOT NULL DEFAULT 1 CHECK(dyn_active IN (0, 1)),
                FOREIGN KEY (uid) 
                REFERENCES subTarget(uid) 
                ON DELETE CASCADE
            );
            """
        await self.__cursor.execute(sql)
        
    async def __creat_botStatus_table(self):
        sql ="""
            CREATE TABLE IF NOT EXISTS botStatus (
                id INTEGER PRIMARY KEY AUTOINCREMENT, 
                bot INTEGER NOT NULL,
                groupId INTEGER NOT NULL,
                push_active INTEGER NOT NULL DEFAULT 1 CHECK(push_active IN (0, 1)),
                bot_active INTEGER NOT NULL DEFAULT 1 CHECK(bot_active IN (0, 1))
            );
            """
        await self.__cursor.execute(sql)
            
    async def __creat_cookie_table(self):
        sql ="""
            CREATE TABLE IF NOT EXISTS Cookie (
                id INTEGER PRIMARY KEY AUTOINCREMENT, 
                cookie  BLOB NOT NULL,
                refresh_token  TEXT NOT NULL
            );
            """
        await self.__cursor.execute(sql)

    async def __creat_dynamic_table(self):
        sql ="""
            CREATE TABLE IF NOT EXISTS Dynamic (
                dyn_id TEXT NOT NULL PRIMARY KEY,
                uname  TEXT NOT NULL,
                dyn_content  BLOB NOT NULL
            );
            """
        await self.__cursor.execute(sql)


#bot status process
    async def select_bot_status(self,data:set) -> Optional[set]:
        """select bot status

        Args:
            data (set): (bot,groupId)

        Returns:
            Optional[set]: (push_active,bot_active)
        """
        sql = "SELECT push_active,bot_active FROM botStatus WHERE bot=? AND groupId=?"
        await self.__cursor.execute(sql,data)
        return await self.__cursor.fetchone()
    
    async def select_all_bot(self) -> Optional[list]:
        """select all bot

        :return: [bot]
        :rtype: Optional[list]
        """
        sql = "SELECT DISTINCT  bot FROM botStatus"
        await self.__cursor.execute(sql)
        return await self.__cursor.fetchall()
    
    async def insert_bot_status(self,data:set):
        """insert bot status

        Args:
            data (set): (bot,groupId,push_active,bot_active)
        """
        sql = "INSERT INTO botStatus (bot,groupId,push_active,bot_active) VALUES (?, ?,?,?);"
        await self.__cursor.execute(sql,data)
        
    async def update_bot_push_status(self,data:set) -> None:
        """update bot push status

        Args:
            data (set): (push_active,bot,groupId)
        """
        sql = "UPDATE botStatus SET push_active = ?  WHERE bot=? AND groupId=?"
        await self.__cursor.execute(sql,data)
        
    async def updata_bot_active_status(self,data:set):
        """update bot active status

        Args:
            data (set): (bot_active, bot)
        """
        sql = "UPDATE botStatus SET bot_active = ? WHERE bot=?"
        await self.__cursor.execute(sql,data)
        

# cookie process
    async def select_cookie(self):
        sql = "SELECT cookie, refresh_token FROM Cookie"
        await self.__cursor.execute(sql)
        return await self.__cursor.fetchone()  
    
    async def insert_cookie(self,data:set):
        sql = "INSERT INTO Cookie (cookie,refresh_token) VALUES (?, ?);"
        await self.__cursor.execute(sql,data)
    
    async def update_cookie(self,data:set):
        sql = "UPDATE Cookie SET cookie = ? , refresh_token=?  WHERE refresh_token=?"
        await self.__cursor.execute(sql,data)

#dynamic process
    async def select_dyn_content(self,dyn_id: str):
        sql = "SELECT dyn_content FROM Dynamic WHERE dyn_id=?"
        await self.__cursor.execute(sql,(dyn_id,))
        return await self.__cursor.fetchone()
    
    async def insert_dyn_data(self,dyn_data:tuple):
        """insert dyn data

        Args:
            dyn_data (tuple): (dyn_id,uname,dyn_content)
        """
        sql = "INSERT INTO Dynamic (dyn_id,uname,dyn_content) VALUES (?, ?,?);"
        await self.__cursor.execute(sql,dyn_data)

# subTarget process
    async def insert_sub_target(self,sub_data:tuple):
        """增加订阅记录

        :param sub_data: (uid,nickname,live_status)
        :type sub_data: tuple
        """
        sql = "INSERT INTO subTarget (uid,nickname,live_status) VALUES (?, ?,?);"
        await self.__cursor.execute(sql,sub_data)
    
    async def select_sub_target(self,uid:str):
        sql = "SELECT nickname FROM subTarget WHERE uid=?"
        await self.__cursor.execute(sql,(uid,))
        return await self.__cursor.fetchone()
    
    async def update_sub_target(self,data:tuple):
        """修改订阅名单

        :param data: (nickname,live_status,uid)
        :type data: tuple
        """
        sql = "UPDATE subTarget SET  nickname=?, live_status=?  WHERE uid=?"
        await self.__cursor.execute(sql,data)

#subChennal process

    async def insert_sub_chennal(self,data:tuple):
        """增加订阅群组及机器人记录

        :param data: (uid,groupId,bot)
        :type data: tuple
        """
        sql = "INSERT INTO subChennal (uid,groupId,bot) VALUES (?, ?,?);"
        await self.__cursor.execute(sql,data)
    
    async def update_sub_chennal(self,data:tuple):
        """修改订阅群组记录

        :param data: (live_active,dyn_active,uid,groupId,bot)
        :type data: tuple
        """
        sql = "UPDATE subChennal SET  live_active=?, dyn_active=?  WHERE uid=? AND groupId=? AND bot=?"
        await self.__cursor.execute(sql,data)
        
    async def select_sub_chennal(self,data:tuple) -> Optional[set]:
        """select sun chennal data

        Args:
            data (tuple): (uid, groupId, bot)

        Returns:
            Optional[set]: 
        """
        sql = "SELECT  live_active, dyn_active FROM subChennal  WHERE uid=? AND groupId=? AND bot=?"
        await self.__cursor.execute(sql,data)
        return await self.__cursor.fetchone()
    
# find dyn push list
    async def select_dynamic_push(self,uid:str) -> Optional[list]:
        """select dynamic push group and bot

        Args:
            uid (str): uid

        Returns:
            Optional[list]: [(groupId,bot)]
        """
        sql = """
            SELECT DISTINCT t2.groupId, t2.bot
            FROM 
                subTarget t1
                INNER JOIN subChennal t2 ON t1.uid = t2.uid
                INNER JOIN botStatus t3 ON t2.groupId = t3.groupId AND t2.bot = t3.bot
            WHERE 
                t1.uid = ? 
                AND t3.push_active = 1
                AND t2.dyn_active = 1
                AND t3.bot_active = 1;
            """
        
        await self.__cursor.execute(sql,(int(uid),))
        return await self.__cursor.fetchall()

#find live check uid
    async def select_live_check_uid(self) -> Optional[list]:
            """select dynamic push group and bot

            Args:
                uid (str): uid

            Returns:
                Optional[list]: [(groupId,bot)]
            """
            sql = """
                SELECT DISTINCT t1.uid,t1.live_status
                FROM 
                    subTarget t1
                    INNER JOIN subChennal t2 ON t1.uid = t2.uid
                    INNER JOIN botStatus t3 ON t2.groupId = t3.groupId AND t2.bot = t3.bot
                WHERE 
                    t3.push_active = 1
                    AND t2.live_active = 1
                    AND t3.bot_active = 1;
                """
            
            await self.__cursor.execute(sql)
            return await self.__cursor.fetchall()
#find live push list
    async def select_live_push(self,uid:str) -> Optional[list]:
            """select live push group and bot

            Args:
                uid (str): uid

            Returns:
                Optional[list]: [(groupId,bot)]
            """
            sql = """
                SELECT DISTINCT t2.groupId, t2.bot
                FROM 
                    subTarget t1
                    INNER JOIN subChennal t2 ON t1.uid = t2.uid
                    INNER JOIN botStatus t3 ON t2.groupId = t3.groupId AND t2.bot = t3.bot
                WHERE 
                    t1.uid = ? 
                    AND t3.push_active = 1
                    AND t2.live_active = 1
                    AND t3.bot_active = 1;
                """
            
            await self.__cursor.execute(sql,(uid,))
            return await self.__cursor.fetchall()

# get sub list
    async def select_sub_list(self,data:set) -> Optional[list]:
        """select sub list data

        Args:
            data (tuple): (bot, groupId)

        Returns:
            Optional[set]: (nickname,live_active,dyn_active)
        """

        sql = """
            SELECT DISTINCT t1.uid, t1.nickname,t2.live_active,t2.dyn_active
            FROM
                subTarget t1
                INNER JOIN subChennal t2 ON t1.uid = t2.uid
            WHERE
                t2.bot=?
                AND t2.groupId=?        
            """
        await self.__cursor.execute(sql,data)
        return await self.__cursor.fetchall()
        
        






import pickle
import skia
import asyncio
from nonebot import logger
from io import BytesIO
from nonebot import get_bot
from dynrender_skia.Core import DynRender
from nonebot.adapters.onebot.v11 import MessageSegment,Bot
from arielbot.plugins.Core.ariel_database import DataManager
from arielbot.plugins.Core.ariel_bili import Dynamic,Live


class PublicPusher:
    async def assign_tasks(self,task):
        await asyncio.gather(*[self.process_task(i,task["message"]) for i in task["target"]])
    
    async def process_task(self,push_target,message):
        bot:Bot = get_bot(str(push_target[1]))
        await bot.send_group_msg(group_id=push_target[0],message=message)


class DynPusher(PublicPusher):
    
    async def push_dynamic(self):
        follow_dynamic_list = await Dynamic().get_dynamic_from_follow_list()
        if follow_dynamic_list is None:
            return
        task_list = []
        async with DataManager() as m:
            for dynamic in follow_dynamic_list:
                result = await m.select_dyn_content(dynamic.message_id)
                if result:
                    continue
                logger.info(f"检测到{dynamic.header.name}的新动态: {dynamic.message_id}")
                all_push_group = await m.select_dynamic_push(dynamic.header.mid)
                await m.insert_dyn_data((dynamic.message_id,dynamic.header.name,pickle.dumps(dynamic)))
                if not all_push_group:
                    logger.info("没有需要推送的群，跳过该动态")
                    continue
                img = await DynRender(font_family="Noto Sans CJK SC").run(dynamic)
                img = skia.Image.fromarray(img, colorType=skia.ColorType.kRGBA_8888_ColorType)
                img_buffer = BytesIO()
                img.save(img_buffer)
                message = MessageSegment.text(f"{dynamic.header.name}发布了新动态:\n\n")+MessageSegment.text(f"传送门→https://t.bilibili.com/{dynamic.message_id}")+MessageSegment.image(img_buffer)
                task_list.append({"target":all_push_group,"message":message})
        if task_list:
            await asyncio.gather(*[self.assign_tasks(i) for i in task_list])
    
    
        
                
                
        
class LivePusher(PublicPusher):
    
    async def push_live(self):
        async with DataManager() as m:
            all_check_uid_list = await m.select_live_check_uid()
        if not all_check_uid_list:
            return
        all_live_stauts = {f"{i[0]}":i[1] for i in all_check_uid_list}
        all_check_uid_list = [i[0] for i in all_check_uid_list]
        check_result= await Live().get_room_info_by_uids(all_check_uid_list)
        if check_result  is None:
            return
        tasks = []
        for k,v in check_result.items():
            if v["live_status"] !=1:
                v["live_status"]=0
            
            if all_live_stauts[k] == v["live_status"]:
                continue
            if all_live_stauts[k] == 1:
                async with DataManager() as m:
                    await m.update_sub_target((v["uname"],0,v["uid"]))
                continue
            async with DataManager() as m:
                await m.update_sub_target((v["uname"],1,v["uid"]))
                all_push_target = await m.select_live_push(v["uid"])
            if not all_push_target:
                continue
            logger.info(f"{v["live_status"]},{all_live_stauts[k]}")
            message = MessageSegment.text(f"【{v["uname"]}】开播啦!!!\n\n标题：{v["title"]}\n\n")+MessageSegment.text(f"传送门：https://live.bilibili.com/{v["room_id"]}\n")+MessageSegment.image(v["cover_from_user"])
            tasks.append({"target":all_push_target,"message":message})
        if tasks:
            await asyncio.gather(*[self.assign_tasks(i) for i in tasks])
    
    
            
            
                
            
        
        
        
        
        

    
    


from nonebot import on_command
from nonebot.adapters import Bot
from nonebot.adapters import Message
from nonebot.params import CommandArg
from nonebot.permission import SUPERUSER
from nonebot.adapters.onebot.v11 import GROUP_ADMIN
from nonebot.adapters.onebot.v11 import GROUP_OWNER
from nonebot.adapters.onebot.v11 import GroupMessageEvent
from arielbot.plugins.Core.ariel_tools import *

from arielbot.plugins.Core.ariel_tools import LoginTools
from arielbot.plugins.Core.ariel_rule import bot_is_active

login = on_command("login", aliases={"登录"}, permission=SUPERUSER)

add_sub = on_command("sub", rule=bot_is_active, aliases={"订阅"}, permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)
del_sub = on_command("unsub",  rule=bot_is_active, aliases={"删除"}, permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)


live_active = on_command("live_on", rule=bot_is_active, permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)
live_deactivate = on_command("live_off", rule=bot_is_active, permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)



dyn_active = on_command("dyn_on", rule=bot_is_active, permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)
dyn_deactivate = on_command("dyn_off", rule=bot_is_active, permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)

bot_active = on_command("bot_on", permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)
bot_deactivate = on_command("bot_off", permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER)

sub_list = on_command("list",  rule=bot_is_active, aliases={"列表"})
bot_help = on_command("help")

@login.handle()
async def _(bot:Bot,event:GroupMessageEvent):
    login_handler = LoginTools()
    await login_handler.login_handle(bot, event)
    await login.finish()

@add_sub.handle()
async def _(event:GroupMessageEvent, args: Message = CommandArg()):
    if args.extract_plain_text() and args.extract_plain_text().isdigit():
        add_sub_processor = AddSubTools(args.extract_plain_text())
        result = await add_sub_processor.add_sub_processor(event)
        await add_sub.finish(result)
    else:
        await add_sub.finish("请携带正确的uid后重试")

@del_sub.handle()
async def _(event:GroupMessageEvent, args: Message = CommandArg()):
    if args.extract_plain_text() and args.extract_plain_text().isdigit():
        del_sub_processor = DelSubTools(args.extract_plain_text())
        result = await del_sub_processor.del_sub_processor(event)
        await del_sub.finish(result)
    else:
        await del_sub.finish("请携带正确的uid后重试")


@live_active.handle()
async def _(event:GroupMessageEvent, args: Message = CommandArg()):
    if args.extract_plain_text() and args.extract_plain_text().isdigit():
        live_active_processor = UpdateSubTools(args.extract_plain_text())
        result = await live_active_processor.update_sub_handler(event,1)
        await live_active.finish(result)
    else:
        await live_active.finish("请携带正确的uid后重试")
    
@live_deactivate.handle()
async def _(event:GroupMessageEvent, args: Message = CommandArg()):
    if args.extract_plain_text() and args.extract_plain_text().isdigit():
        live_deactivate_processor = UpdateSubTools(args.extract_plain_text())
        result = await live_deactivate_processor.update_sub_handler(event,0)
        await live_deactivate.finish(result)
    else:
        await live_deactivate.finish("请携带正确的uid后重试")

@dyn_active.handle()
async def _(event:GroupMessageEvent, args: Message = CommandArg()):
    if args.extract_plain_text() and args.extract_plain_text().isdigit():
        dyn_active_processor = UpdateSubTools(args.extract_plain_text())
        result = await dyn_active_processor.update_sub_handler(event,dyn_active=1)
        await dyn_active.finish(result)
    else:
        await dyn_active.finish("请携带正确的uid后重试")
        
@dyn_deactivate.handle()
async def _(event:GroupMessageEvent, args: Message = CommandArg()):
    if args.extract_plain_text() and args.extract_plain_text().isdigit():
        dyn_deactivate_processor = UpdateSubTools(args.extract_plain_text())
        result = await dyn_deactivate_processor.update_sub_handler(event,dyn_active=0)
        await dyn_deactivate.finish(result)
    else:
        await dyn_deactivate.finish("请携带正确的uid后重试")

@bot_active.handle()
async def _(event:GroupMessageEvent):
    bot_active_processor = UpdateBotStatusTools()
    result = await bot_active_processor.update_bot_status_processor(event,1)
    await bot_active.finish(result)

@bot_deactivate.handle()
async def _(event:GroupMessageEvent):
    bot_deactivate_processor = UpdateBotStatusTools()
    result = await bot_deactivate_processor.update_bot_status_processor(event,0)
    if result is None:
        await bot_deactivate.finish()
    else:
        await bot_deactivate.finish(result)

@bot_help.handle()
async def _():
    await bot_help.finish(MessageSegment.image("https://i0.hdslb.com/bfs/new_dyn/abef945ad1d209ad1d2360624180a15d490040351.png"))

@sub_list.handle()
async def _(event:GroupMessageEvent):
    sub_list_processor = SubListTools()
    img = await sub_list_processor.get_sub_list_data(event)
    await sub_list.finish(img)
    




from arielbot.plugins.Core.ariel_cmd import *

from nonebot import get_driver

from nonebot import require

require("nonebot_plugin_apscheduler")

from nonebot_plugin_apscheduler import scheduler

from arielbot.plugins.Core.ariel_push import DynPusher,LivePusher

driver = get_driver()

@driver.on_bot_connect
async def _(bot: Bot):
    await UpdateBotStatusTools.update_bot_active_processor((1,bot.self_id))
    
@driver.on_bot_disconnect
async def _(bot: Bot):
    await UpdateBotStatusTools.update_bot_active_processor((0,bot.self_id))
    
@driver.on_shutdown
async def _():
    await UpdateBotStatusTools.shutdown_all_bot()


# 基于装饰器的方式
@scheduler.scheduled_job("cron", second="*/8", id="dyn_pusher",max_instances=1)
async def _():
    pusher = DynPusher()
    await pusher.push_dynamic()


@scheduler.scheduled_job("cron", second="*/10", id="live_pusher",max_instances=1)
async def _():
    pusher = LivePusher()
    await pusher.push_live()


import click
from os import path, getcwd, mkdir
import dotenv
import nonebot


env = {
    'DRIVER':'~fastapi',
    'SUPERUSERS': [],
    'HOST':"127.0.0.1",
    'PORT':12315,
    'COMMAND_START':["/"]
}


@click.group()
def main():
    pass


@click.command()
def run():
    create_config()
    create_plugins_dir()
    from .bot import app
    nonebot.run(app=app)


main.add_command(run)


def create_config():
    env_file_path = path.join(getcwd(), ".env.prod")
    if not path.exists(env_file_path):
        for key, value in env.items():
            dotenv.set_key(
                env_file_path,
                key,
                str(value).replace(' ', ''),
                quote_mode="never"
            )


def create_plugins_dir():
    plugins_dir_path = path.join(getcwd(), "plugins")
    if not path.exists(plugins_dir_path):
        mkdir(plugins_dir_path)


